@@ -11,7 +11,8 @@ pub struct Options {
 }
 
 pub const USAGE: &'static str = "
-Print a JSON representation of a Cargo.toml manifest
+Deprecated, use `cargo metadata --no-deps` instead.
+Print a JSON representation of a Cargo.toml manifest.
 
 Usage:
     cargo read-manifest [options]
@@ -19,7 +20,7 @@ Usage:
 
 Options:
     -h, --help               Print this message
-    -v, --verbose            Use verbose output
+    -v, --verbose ...        Use verbose output (-vv very verbose/build.rs output)
     --manifest-path PATH     Path to the manifest
     --color WHEN             Coloring: auto, always, never
 ";
@@ -27,10 +28,10 @@ Options:
 pub fn execute(options: Options, config: &Config) -> CliResult<Option<Package>> {
     debug!("executing; cmd=cargo-read-manifest; args={:?}",
            env::args().collect::<Vec<_>>());
-    try!(config.shell().set_color_config(options.flag_color.as_ref().map(|s| &s[..])));
+    config.shell().set_color_config(options.flag_color.as_ref().map(|s| &s[..]))?;
 
-    let root = try!(find_root_manifest_for_wd(options.flag_manifest_path, config.cwd()));
+    let root = find_root_manifest_for_wd(options.flag_manifest_path, config.cwd())?;
 
-    let pkg = try!(Package::for_path(&root, config));
+    let pkg = Package::for_path(&root, config)?;
     Ok(Some(pkg))
 }
