@@ -5,9 +5,11 @@ use cargo::util::{CliResult, Config};
 pub struct Options {
     flag_bin: Vec<String>,
     flag_root: Option<String>,
-    flag_verbose: Option<bool>,
+    flag_verbose: u32,
     flag_quiet: Option<bool>,
     flag_color: Option<String>,
+    flag_frozen: bool,
+    flag_locked: bool,
 
     arg_spec: String,
 }
@@ -23,9 +25,11 @@ Options:
     -h, --help                Print this message
     --root DIR                Directory to uninstall packages from
     --bin NAME                Only uninstall the binary NAME
-    -v, --verbose             Use verbose output
+    -v, --verbose ...         Use verbose output (-vv very verbose/build.rs output)
     -q, --quiet               Less output printed to stdout
     --color WHEN              Coloring: auto, always, never
+    --frozen                  Require Cargo.lock and cache are up to date
+    --locked                  Require Cargo.lock is up to date
 
 The argument SPEC is a package id specification (see `cargo help pkgid`) to
 specify which crate should be uninstalled. By default all binaries are
@@ -34,12 +38,14 @@ only uninstall particular binaries.
 ";
 
 pub fn execute(options: Options, config: &Config) -> CliResult<Option<()>> {
-    try!(config.configure_shell(options.flag_verbose,
-                                options.flag_quiet,
-                                &options.flag_color));
+    config.configure(options.flag_verbose,
+                     options.flag_quiet,
+                     &options.flag_color,
+                     options.flag_frozen,
+                     options.flag_locked)?;
 
     let root = options.flag_root.as_ref().map(|s| &s[..]);
-    try!(ops::uninstall(root, &options.arg_spec, &options.flag_bin, config));
+    ops::uninstall(root, &options.arg_spec, &options.flag_bin, config)?;
     Ok(None)
 }
 
