@@ -1,13 +1,15 @@
 pub use self::dependency::{Dependency, DependencyInner};
 pub use self::manifest::{Manifest, Target, TargetKind, Profile, LibKind, Profiles};
+pub use self::manifest::{EitherManifest, VirtualManifest};
 pub use self::package::{Package, PackageSet};
-pub use self::package_id::{PackageId, Metadata};
+pub use self::package_id::PackageId;
 pub use self::package_id_spec::PackageIdSpec;
 pub use self::registry::Registry;
 pub use self::resolver::Resolve;
 pub use self::shell::{Shell, MultiShell, ShellConfig, Verbosity, ColorConfig};
 pub use self::source::{Source, SourceId, SourceMap, GitReference};
 pub use self::summary::Summary;
+pub use self::workspace::{Workspace, WorkspaceConfig};
 
 pub mod source;
 pub mod package;
@@ -19,3 +21,4 @@ pub mod summary;
 pub mod shell;
 pub mod registry;
 mod package_id_spec;
+mod workspace;
