@@ -23,18 +23,13 @@
 //!
 
 use std::collections::HashMap;
-use std::default::Default;
-use std::path::{Path, PathBuf};
-use std::sync::Arc;
+use std::path::PathBuf;
 
-use core::registry::PackageRegistry;
-use core::{Source, SourceId, PackageSet, Package, Target};
-use core::{Profile, TargetKind, Profiles};
-use core::resolver::{Method, Resolve};
-use ops::{self, BuildOutput, ExecEngine};
-use sources::PathSource;
+use core::{Source, Package, Target};
+use core::{Profile, TargetKind, Profiles, Workspace, PackageIdSpec};
+use ops::{self, BuildOutput};
 use util::config::Config;
-use util::{CargoResult, profile, human, ChainError};
+use util::{CargoResult, profile};
 
 /// Contains information about how a package should be compiled.
 pub struct CompileOptions<'a> {
@@ -45,19 +40,21 @@ pub struct CompileOptions<'a> {
     pub target: Option<&'a str>,
     /// Extra features to build for the root package
     pub features: &'a [String],
+    /// Flag whether all available features should be built for the root package
+    pub all_features: bool,
     /// Flag if the default feature should be built for the root package
     pub no_default_features: bool,
-    /// Root package to build (if None it's the current one)
-    pub spec: &'a [String],
+    /// A set of packages to build.
+    pub spec: Packages<'a>,
     /// Filter to apply to the root package to select which targets will be
     /// built.
     pub filter: CompileFilter<'a>,
-    /// Engine which drives compilation
-    pub exec_engine: Option<Arc<Box<ExecEngine>>>,
     /// Whether this is a release build or not
     pub release: bool,
     /// Mode for this compile.
     pub mode: CompileMode,
+    /// `--error_format` flag for the compiler.
+    pub message_format: MessageFormat,
     /// Extra arguments to be passed to rustdoc (for main crate and dependencies)
     pub target_rustdoc_args: Option<&'a [String]>,
     /// The specified target will be compiled with all the available arguments,
@@ -65,14 +62,44 @@ pub struct CompileOptions<'a> {
     pub target_rustc_args: Option<&'a [String]>,
 }
 
-#[derive(Clone, Copy, PartialEq)]
+#[derive(Clone, Copy, PartialEq, Debug)]
 pub enum CompileMode {
     Test,
     Build,
+    Check,
     Bench,
     Doc { deps: bool },
 }
 
+#[derive(Clone, Copy, PartialEq, Eq, RustcDecodable)]
+pub enum MessageFormat {
+    Human,
+    Json
+}
+
+#[derive(Clone, Copy, PartialEq, Eq)]
+pub enum Packages<'a> {
+    All,
+    Packages(&'a [String]),
+}
+
+impl<'a> Packages<'a> {
+    pub fn into_package_id_specs(self, ws: &Workspace) -> CargoResult<Vec<PackageIdSpec>> {
+        let specs = match self {
+            Packages::All => {
+                ws.members()
+                    .map(Package::package_id)
+                    .map(PackageIdSpec::from_package_id)
+                    .collect()
+            }
+            Packages::Packages(packages) => {
+                packages.iter().map(|p| PackageIdSpec::parse(&p)).collect::<CargoResult<Vec<_>>>()?
+            }
+        };
+        Ok(specs)
+    }
+}
+
 pub enum CompileFilter<'a> {
     Everything,
     Only {
@@ -84,101 +111,58 @@ pub enum CompileFilter<'a> {
     }
 }
 
-pub fn compile<'a>(manifest_path: &Path,
-                   options: &CompileOptions<'a>)
+pub fn compile<'a>(ws: &Workspace<'a>, options: &CompileOptions<'a>)
                    -> CargoResult<ops::Compilation<'a>> {
-    debug!("compile; manifest-path={}", manifest_path.display());
-
-    let package = try!(Package::for_path(manifest_path, options.config));
-    debug!("loaded package; package={}", package);
-
-    for key in package.manifest().warnings().iter() {
-        try!(options.config.shell().warn(key))
+    for member in ws.members() {
+        for key in member.manifest().warnings().iter() {
+            options.config.shell().warn(key)?
+        }
     }
-    compile_pkg(&package, None, options)
+    compile_ws(ws, None, options)
 }
 
-pub fn resolve_dependencies<'a>(root_package: &Package,
-                                config: &'a Config,
-                                source: Option<Box<Source + 'a>>,
-                                features: Vec<String>,
-                                no_default_features: bool)
-                                -> CargoResult<(PackageSet<'a>, Resolve)> {
-
-    let mut registry = PackageRegistry::new(config);
-
-    if let Some(source) = source {
-        registry.add_preloaded(root_package.package_id().source_id(), source);
-    }
-
-    // First, resolve the root_package's *listed* dependencies, as well as
-    // downloading and updating all remotes and such.
-    let resolve = try!(ops::resolve_pkg(&mut registry, root_package, config));
-
-    // Second, resolve with precisely what we're doing. Filter out
-    // transitive dependencies if necessary, specify features, handle
-    // overrides, etc.
-    let _p = profile::start("resolving w/ overrides...");
-
-    try!(add_overrides(&mut registry, root_package.root(), config));
-
-    let method = Method::Required{
-        dev_deps: true, // TODO: remove this option?
-        features: &features,
-        uses_default_features: !no_default_features,
-    };
-
-    let resolved_with_overrides =
-            try!(ops::resolve_with_previous(&mut registry, root_package,
-                                            method, Some(&resolve), None));
-
-    let packages = ops::get_resolved_packages(&resolved_with_overrides,
-                                              registry);
-
-    Ok((packages, resolved_with_overrides))
-}
-
-pub fn compile_pkg<'a>(root_package: &Package,
-                       source: Option<Box<Source + 'a>>,
-                       options: &CompileOptions<'a>)
-                       -> CargoResult<ops::Compilation<'a>> {
+pub fn compile_ws<'a>(ws: &Workspace<'a>,
+                      source: Option<Box<Source + 'a>>,
+                      options: &CompileOptions<'a>)
+                      -> CargoResult<ops::Compilation<'a>> {
     let CompileOptions { config, jobs, target, spec, features,
-                         no_default_features, release, mode,
-                         ref filter, ref exec_engine,
+                         all_features, no_default_features,
+                         release, mode, message_format,
+                         ref filter,
                          ref target_rustdoc_args,
                          ref target_rustc_args } = *options;
 
     let target = target.map(|s| s.to_string());
-    let features = features.iter().flat_map(|s| {
-        s.split(' ')
-    }).map(|s| s.to_string()).collect::<Vec<String>>();
 
     if jobs == Some(0) {
         bail!("jobs must be at least 1")
     }
 
-    let profiles = root_package.manifest().profiles();
-    if spec.len() == 0 {
-        try!(generate_targets(root_package, profiles, mode, filter, release));
-    }
+    let profiles = ws.profiles();
 
-    let (packages, resolve_with_overrides) = {
-        try!(resolve_dependencies(root_package, config, source, features,
-                                  no_default_features))
-    };
+    let specs = spec.into_package_id_specs(ws)?;
+    let resolve = ops::resolve_ws_precisely(ws,
+                                            source,
+                                            features,
+                                            all_features,
+                                            no_default_features,
+                                            &specs)?;
+    let (packages, resolve_with_overrides) = resolve;
 
     let mut pkgids = Vec::new();
-    if spec.len() > 0 {
-        for p in spec {
-            pkgids.push(try!(resolve_with_overrides.query(&p)));
+    if specs.len() > 0 {
+        for p in specs.iter() {
+            pkgids.push(p.query(resolve_with_overrides.iter())?);
         }
     } else {
+        let root_package = ws.current()?;
+        generate_targets(root_package, profiles, mode, filter, release)?;
         pkgids.push(root_package.package_id());
     };
 
-    let to_builds = try!(pkgids.iter().map(|id| {
+    let to_builds = pkgids.iter().map(|id| {
         packages.get(id)
-    }).collect::<CargoResult<Vec<_>>>());
+    }).collect::<CargoResult<Vec<_>>>()?;
 
     let mut general_targets = Vec::new();
     let mut package_targets = Vec::new();
@@ -189,8 +173,8 @@ pub fn compile_pkg<'a>(root_package: &Package,
             panic!("`rustc` and `rustdoc` should not accept multiple `-p` flags")
         }
         (Some(args), _) => {
-            let targets = try!(generate_targets(to_builds[0], profiles,
-                                                mode, filter, release));
+            let targets = generate_targets(to_builds[0], profiles,
+                                           mode, filter, release)?;
             if targets.len() == 1 {
                 let (target, profile) = targets[0];
                 let mut profile = profile.clone();
@@ -203,8 +187,8 @@ pub fn compile_pkg<'a>(root_package: &Package,
             }
         }
         (None, Some(args)) => {
-            let targets = try!(generate_targets(to_builds[0], profiles,
-                                                mode, filter, release));
+            let targets = generate_targets(to_builds[0], profiles,
+                                           mode, filter, release)?;
             if targets.len() == 1 {
                 let (target, profile) = targets[0];
                 let mut profile = profile.clone();
@@ -218,8 +202,8 @@ pub fn compile_pkg<'a>(root_package: &Package,
         }
         (None, None) => {
             for &to_build in to_builds.iter() {
-                let targets = try!(generate_targets(to_build, profiles, mode,
-                                                    filter, release));
+                let targets = generate_targets(to_build, profiles, mode,
+                                               filter, release)?;
                 package_targets.push((to_build, targets));
             }
         }
@@ -233,21 +217,21 @@ pub fn compile_pkg<'a>(root_package: &Package,
 
     let mut ret = {
         let _p = profile::start("compiling");
-        let mut build_config = try!(scrape_build_config(config, jobs, target));
-        build_config.exec_engine = exec_engine.clone();
+        let mut build_config = scrape_build_config(config, jobs, target)?;
         build_config.release = release;
-        build_config.test = mode == CompileMode::Test;
+        build_config.test = mode == CompileMode::Test || mode == CompileMode::Bench;
+        build_config.json_messages = message_format == MessageFormat::Json;
         if let CompileMode::Doc { deps } = mode {
             build_config.doc_all = deps;
         }
 
-        try!(ops::compile_targets(&package_targets,
-                                  &packages,
-                                  &resolve_with_overrides,
-                                  config,
-                                  build_config,
-                                  root_package.manifest().profiles(),
-                                  ))
+        ops::compile_targets(ws,
+                             &package_targets,
+                             &packages,
+                             &resolve_with_overrides,
+                             config,
+                             build_config,
+                             profiles)?
     };
 
     ret.to_doc_test = to_builds.iter().map(|&p| p.clone()).collect();
@@ -304,6 +288,7 @@ fn generate_targets<'a>(pkg: &'a Package,
         CompileMode::Test => test,
         CompileMode::Bench => &profiles.bench,
         CompileMode::Build => build,
+        CompileMode::Check => &profiles.check,
         CompileMode::Doc { .. } => &profiles.doc,
     };
     match *filter {
@@ -315,22 +300,27 @@ fn generate_targets<'a>(pkg: &'a Package,
                     }).collect::<Vec<_>>())
                 }
                 CompileMode::Test => {
+                    let deps = if release {
+                        &profiles.bench_deps
+                    } else {
+                        &profiles.test_deps
+                    };
                     let mut base = pkg.targets().iter().filter(|t| {
                         t.tested()
                     }).map(|t| {
-                        (t, if t.is_example() {build} else {profile})
+                        (t, if t.is_example() {deps} else {profile})
                     }).collect::<Vec<_>>();
 
                     // Always compile the library if we're testing everything as
                     // it'll be needed for doctests
                     if let Some(t) = pkg.targets().iter().find(|t| t.is_lib()) {
                         if t.doctested() {
-                            base.push((t, build));
+                            base.push((t, deps));
                         }
                     }
                     Ok(base)
                 }
-                CompileMode::Build => {
+                CompileMode::Build | CompileMode::Check => {
                     Ok(pkg.targets().iter().filter(|t| {
                         t.is_bin() || t.is_lib()
                     }).map(|t| (t, profile)).collect())
@@ -377,49 +367,16 @@ fn generate_targets<'a>(pkg: &'a Package,
                     }
                     Ok(())
                 };
-                try!(find(bins, "bin", TargetKind::Bin, profile));
-                try!(find(examples, "example", TargetKind::Example, build));
-                try!(find(tests, "test", TargetKind::Test, test));
-                try!(find(benches, "bench", TargetKind::Bench, &profiles.bench));
+                find(bins, "bin", TargetKind::Bin, profile)?;
+                find(examples, "example", TargetKind::Example, build)?;
+                find(tests, "test", TargetKind::Test, test)?;
+                find(benches, "bench", TargetKind::Bench, &profiles.bench)?;
             }
             Ok(targets)
         }
     }
 }
 
-/// Read the `paths` configuration variable to discover all path overrides that
-/// have been configured.
-fn add_overrides<'a>(registry: &mut PackageRegistry<'a>,
-                     cur_path: &Path,
-                     config: &'a Config) -> CargoResult<()> {
-    let paths = match try!(config.get_list("paths")) {
-        Some(list) => list,
-        None => return Ok(())
-    };
-    let paths = paths.val.iter().map(|&(ref s, ref p)| {
-        // The path listed next to the string is the config file in which the
-        // key was located, so we want to pop off the `.cargo/config` component
-        // to get the directory containing the `.cargo` folder.
-        (p.parent().unwrap().parent().unwrap().join(s), p)
-    }).filter(|&(ref p, _)| {
-        // Make sure we don't override the local package, even if it's in the
-        // list of override paths.
-        cur_path != &**p
-    });
-
-    for (path, definition) in paths {
-        let id = try!(SourceId::for_path(&path));
-        let mut source = PathSource::new_recursive(&path, &id, config);
-        try!(source.update().chain_error(|| {
-            human(format!("failed to update path override `{}` \
-                           (defined in `{}`)", path.display(),
-                          definition.display()))
-        }));
-        registry.add_override(&id, Box::new(source));
-    }
-    Ok(())
-}
-
 /// Parse all config files to learn about build configuration. Currently
 /// configured options are:
 ///
@@ -432,7 +389,7 @@ fn scrape_build_config(config: &Config,
                        jobs: Option<u32>,
                        target: Option<String>)
                        -> CargoResult<ops::BuildConfig> {
-    let cfg_jobs = match try!(config.get_i64("build.jobs")) {
+    let cfg_jobs = match config.get_i64("build.jobs")? {
         Some(v) => {
             if v.val <= 0 {
                 bail!("build.jobs must be positive, but found {} in {}",
@@ -447,16 +404,17 @@ fn scrape_build_config(config: &Config,
         None => None,
     };
     let jobs = jobs.or(cfg_jobs).unwrap_or(::num_cpus::get() as u32);
-    let cfg_target = try!(config.get_string("build.target")).map(|s| s.val);
+    let cfg_target = config.get_string("build.target")?.map(|s| s.val);
     let target = target.or(cfg_target);
     let mut base = ops::BuildConfig {
-        jobs: jobs,
+        host_triple: config.rustc()?.host.clone(),
         requested_target: target.clone(),
+        jobs: jobs,
         ..Default::default()
     };
-    base.host = try!(scrape_target_config(config, &config.rustc_info().host));
+    base.host = scrape_target_config(config, &base.host_triple)?;
     base.target = match target.as_ref() {
-        Some(triple) => try!(scrape_target_config(config, &triple)),
+        Some(triple) => scrape_target_config(config, &triple)?,
         None => base.host.clone(),
     };
     Ok(base)
@@ -467,15 +425,15 @@ fn scrape_target_config(config: &Config, triple: &str)
 
     let key = format!("target.{}", triple);
     let mut ret = ops::TargetConfig {
-        ar: try!(config.get_path(&format!("{}.ar", key))).map(|v| v.val),
-        linker: try!(config.get_path(&format!("{}.linker", key))).map(|v| v.val),
+        ar: config.get_path(&format!("{}.ar", key))?.map(|v| v.val),
+        linker: config.get_path(&format!("{}.linker", key))?.map(|v| v.val),
         overrides: HashMap::new(),
     };
-    let table = match try!(config.get_table(&key)) {
+    let table = match config.get_table(&key)? {
         Some(table) => table.val,
         None => return Ok(ret),
     };
-    for (lib_name, _) in table.into_iter() {
+    for (lib_name, value) in table {
         if lib_name == "ar" || lib_name == "linker" || lib_name == "rustflags" {
             continue
         }
@@ -486,40 +444,39 @@ fn scrape_target_config(config: &Config, triple: &str)
             cfgs: Vec::new(),
             metadata: Vec::new(),
             rerun_if_changed: Vec::new(),
+            warnings: Vec::new(),
         };
-        let key = format!("{}.{}", key, lib_name);
-        let table = try!(config.get_table(&key)).unwrap().val;
-        for (k, _) in table.into_iter() {
+        for (k, value) in value.table(&lib_name)?.0 {
             let key = format!("{}.{}", key, k);
             match &k[..] {
                 "rustc-flags" => {
-                    let flags = try!(config.get_string(&key)).unwrap();
+                    let (flags, definition) = value.string(&k)?;
                     let whence = format!("in `{}` (in {})", key,
-                                         flags.definition);
-                    let (paths, links) = try!(
-                        BuildOutput::parse_rustc_flags(&flags.val, &whence)
-                    );
-                    output.library_paths.extend(paths.into_iter());
-                    output.library_links.extend(links.into_iter());
+                                         definition.display());
+                    let (paths, links) =
+                        BuildOutput::parse_rustc_flags(&flags, &whence)
+                    ?;
+                    output.library_paths.extend(paths);
+                    output.library_links.extend(links);
                 }
                 "rustc-link-lib" => {
-                    let list = try!(config.get_list(&key)).unwrap();
-                    output.library_links.extend(list.val.into_iter()
-                                                        .map(|v| v.0));
+                    let list = value.list(&k)?;
+                    output.library_links.extend(list.iter()
+                                                    .map(|v| v.0.clone()));
                 }
                 "rustc-link-search" => {
-                    let list = try!(config.get_list(&key)).unwrap();
-                    output.library_paths.extend(list.val.into_iter().map(|v| {
+                    let list = value.list(&k)?;
+                    output.library_paths.extend(list.iter().map(|v| {
                         PathBuf::from(&v.0)
                     }));
                 }
                 "rustc-cfg" => {
-                    let list = try!(config.get_list(&key)).unwrap();
-                    output.cfgs.extend(list.val.into_iter().map(|v| v.0));
+                    let list = value.list(&k)?;
+                    output.cfgs.extend(list.iter().map(|v| v.0.clone()));
                 }
                 _ => {
-                    let val = try!(config.get_string(&key)).unwrap();
-                    output.metadata.push((k, val.val));
+                    let val = value.string(&k)?.0;
+                    output.metadata.push((k.clone(), val.to_string()));
                 }
             }
         }
