@@ -7,13 +7,14 @@ use std::io::prelude::*;
 use std::io::SeekFrom;
 use std::path::{Path, PathBuf};
 
+use semver::Version;
 use tempdir::TempDir;
 use toml;
 
 use core::{SourceId, Source, Package, Dependency, PackageIdSpec};
-use core::PackageId;
+use core::{PackageId, Workspace};
 use ops::{self, CompileFilter};
-use sources::{GitSource, PathSource, RegistrySource};
+use sources::{GitSource, PathSource, SourceConfigMap};
 use util::{CargoResult, ChainError, Config, human, internal};
 use util::{Filesystem, FileLock};
 
@@ -53,82 +54,88 @@ pub fn install(root: Option<&str>,
                opts: &ops::CompileOptions,
                force: bool) -> CargoResult<()> {
     let config = opts.config;
-    let root = try!(resolve_root(root, config));
+    let root = resolve_root(root, config)?;
+    let map = SourceConfigMap::new(config)?;
     let (pkg, source) = if source_id.is_git() {
-        try!(select_pkg(GitSource::new(source_id, config), source_id,
-                        krate, vers, &mut |git| git.read_packages()))
+        select_pkg(GitSource::new(source_id, config),
+                   krate, vers, config, &mut |git| git.read_packages())?
     } else if source_id.is_path() {
         let path = source_id.url().to_file_path().ok()
                             .expect("path sources must have a valid path");
         let mut src = PathSource::new(&path, source_id, config);
-        try!(src.update().chain_error(|| {
+        src.update().chain_error(|| {
             human(format!("`{}` is not a crate root; specify a crate to \
                            install from crates.io, or use --path or --git to \
                            specify an alternate source", path.display()))
-        }));
-        try!(select_pkg(PathSource::new(&path, source_id, config),
-                        source_id, krate, vers,
-                        &mut |path| path.read_packages()))
+        })?;
+        select_pkg(PathSource::new(&path, source_id, config),
+                   krate, vers, config, &mut |path| path.read_packages())?
     } else {
-        try!(select_pkg(RegistrySource::new(source_id, config),
-                        source_id, krate, vers,
-                        &mut |_| Err(human("must specify a crate to install from \
-                                            crates.io, or use --path or --git to \
-                                            specify alternate source"))))
+        select_pkg(map.load(source_id)?,
+                   krate, vers, config,
+                   &mut |_| Err(human("must specify a crate to install from \
+                                       crates.io, or use --path or --git to \
+                                       specify alternate source")))?
     };
 
+
+    let mut td_opt = None;
+    let overidden_target_dir = if source_id.is_path() {
+        None
+    } else if let Ok(td) = TempDir::new("cargo-install") {
+        let p = td.path().to_owned();
+        td_opt = Some(td);
+        Some(Filesystem::new(p))
+    } else {
+        Some(Filesystem::new(config.cwd().join("target-install")))
+    };
+
+    let ws = match overidden_target_dir {
+        Some(dir) => Workspace::ephemeral(pkg, config, Some(dir))?,
+        None => Workspace::new(pkg.manifest_path(), config)?,
+    };
+    let pkg = ws.current()?;
+
     // Preflight checks to check up front whether we'll overwrite something.
     // We have to check this again afterwards, but may as well avoid building
     // anything if we're gonna throw it away anyway.
     {
-        let metadata = try!(metadata(config, &root));
-        let list = try!(read_crate_list(metadata.file()));
+        let metadata = metadata(config, &root)?;
+        let list = read_crate_list(metadata.file())?;
         let dst = metadata.parent().join("bin");
-        try!(check_overwrites(&dst, &pkg, &opts.filter, &list, force));
+        check_overwrites(&dst, pkg, &opts.filter, &list, force)?;
     }
 
-    let mut td_opt = None;
-    let target_dir = if source_id.is_path() {
-        config.target_dir(&pkg)
-    } else {
-        if let Ok(td) = TempDir::new("cargo-install") {
-            let p = td.path().to_owned();
-            td_opt = Some(td);
-            Filesystem::new(p)
-        } else {
-            Filesystem::new(config.cwd().join("target-install"))
-        }
-    };
-    config.set_target_dir(target_dir.clone());
-    let compile = try!(ops::compile_pkg(&pkg, Some(source), opts).chain_error(|| {
+    let compile = ops::compile_ws(&ws, Some(source), opts).chain_error(|| {
         if let Some(td) = td_opt.take() {
             // preserve the temporary directory, so the user can inspect it
             td.into_path();
         }
 
         human(format!("failed to compile `{}`, intermediate artifacts can be \
-                       found at `{}`", pkg, target_dir.display()))
-    }));
-    let binaries: Vec<(&str, &Path)> = try!(compile.binaries.iter().map(|bin| {
+                       found at `{}`", pkg, ws.target_dir().display()))
+    })?;
+    let binaries: Vec<(&str, &Path)> = compile.binaries.iter().map(|bin| {
         let name = bin.file_name().unwrap();
         if let Some(s) = name.to_str() {
             Ok((s, bin.as_ref()))
         } else {
             bail!("Binary `{:?}` name can't be serialized into string", name)
         }
-    }).collect::<CargoResult<_>>());
+    }).collect::<CargoResult<_>>()?;
 
-    let metadata = try!(metadata(config, &root));
-    let mut list = try!(read_crate_list(metadata.file()));
+    let metadata = metadata(config, &root)?;
+    let mut list = read_crate_list(metadata.file())?;
     let dst = metadata.parent().join("bin");
-    let duplicates = try!(check_overwrites(&dst, &pkg, &opts.filter, &list, force));
+    let duplicates = check_overwrites(&dst, pkg, &opts.filter,
+                                           &list, force)?;
 
-    try!(fs::create_dir_all(&dst));
+    fs::create_dir_all(&dst)?;
 
     // Copy all binaries to a temporary directory under `dst` first, catching
     // some failure modes (e.g. out of space) before touching the existing
     // binaries. This directory will get cleaned up via RAII.
-    let staging_dir = try!(TempDir::new_in(&dst, "cargo-install"));
+    let staging_dir = TempDir::new_in(&dst, "cargo-install")?;
     for &(bin, src) in binaries.iter() {
         let dst = staging_dir.path().join(bin);
         // Try to move if `target_dir` is transient.
@@ -137,10 +144,10 @@ pub fn install(root: Option<&str>,
                 continue
             }
         }
-        try!(fs::copy(src, &dst).chain_error(|| {
+        fs::copy(src, &dst).chain_error(|| {
             human(format!("failed to copy `{}` to `{}`", src.display(),
                           dst.display()))
-        }));
+        })?;
     }
 
     let (to_replace, to_install): (Vec<&str>, Vec<&str>) =
@@ -153,11 +160,11 @@ pub fn install(root: Option<&str>,
     for bin in to_install.iter() {
         let src = staging_dir.path().join(bin);
         let dst = dst.join(bin);
-        try!(config.shell().status("Installing", dst.display()));
-        try!(fs::rename(&src, &dst).chain_error(|| {
+        config.shell().status("Installing", dst.display())?;
+        fs::rename(&src, &dst).chain_error(|| {
             human(format!("failed to move `{}` to `{}`", src.display(),
                           dst.display()))
-        }));
+        })?;
         installed.bins.push(dst);
     }
 
@@ -169,11 +176,11 @@ pub fn install(root: Option<&str>,
             for &bin in to_replace.iter() {
                 let src = staging_dir.path().join(bin);
                 let dst = dst.join(bin);
-                try!(config.shell().status("Replacing", dst.display()));
-                try!(fs::rename(&src, &dst).chain_error(|| {
+                config.shell().status("Replacing", dst.display())?;
+                fs::rename(&src, &dst).chain_error(|| {
                     human(format!("failed to move `{}` to `{}`", src.display(),
                                   dst.display()))
-                }));
+                })?;
                 replaced_names.push(bin);
             }
             Ok(())
@@ -212,8 +219,8 @@ pub fn install(root: Option<&str>,
     match write_result {
         // Replacement error (if any) isn't actually caused by write error
         // but this seems to be the only way to show both.
-        Err(err) => try!(result.chain_error(|| err)),
-        Ok(_) => try!(result),
+        Err(err) => result.chain_error(|| err)?,
+        Ok(_) => result?,
     }
 
     // Reaching here means all actions have succeeded. Clean up.
@@ -221,8 +228,8 @@ pub fn install(root: Option<&str>,
     if !source_id.is_path() {
         // Don't bother grabbing a lock as we're going to blow it all away
         // anyway.
-        let target_dir = target_dir.into_path_unlocked();
-        try!(fs::remove_dir_all(&target_dir));
+        let target_dir = ws.target_dir().into_path_unlocked();
+        fs::remove_dir_all(&target_dir)?;
     }
 
     // Print a warning that if this directory isn't in PATH that they won't be
@@ -234,50 +241,70 @@ pub fn install(root: Option<&str>,
         }
     }
 
-    try!(config.shell().warn(&format!("be sure to add `{}` to your PATH to be \
+    config.shell().warn(&format!("be sure to add `{}` to your PATH to be \
                                        able to run the installed binaries",
-                                      dst.display())));
+                                      dst.display()))?;
     Ok(())
 }
 
 fn select_pkg<'a, T>(mut source: T,
-                     source_id: &SourceId,
                      name: Option<&str>,
                      vers: Option<&str>,
+                     config: &Config,
                      list_all: &mut FnMut(&mut T) -> CargoResult<Vec<Package>>)
                      -> CargoResult<(Package, Box<Source + 'a>)>
     where T: Source + 'a
 {
-    try!(source.update());
+    source.update()?;
     match name {
         Some(name) => {
-            let dep = try!(Dependency::parse(name, vers, source_id));
-            let deps = try!(source.query(&dep));
+            let vers = match vers {
+                Some(v) => {
+                    match v.parse::<Version>() {
+                        Ok(v) => Some(format!("={}", v)),
+                        Err(_) => {
+                            let msg = format!("the `--vers` provided, `{}`, is \
+                                               not a valid semver version\n\n\
+                                               historically Cargo treated this \
+                                               as a semver version requirement \
+                                               accidentally\nand will continue \
+                                               to do so, but this behavior \
+                                               will be removed eventually", v);
+                            config.shell().warn(&msg)?;
+                            Some(v.to_string())
+                        }
+                    }
+                }
+                None => None,
+            };
+            let vers = vers.as_ref().map(|s| &**s);
+            let dep = Dependency::parse_no_deprecated(name, vers, source.source_id())?;
+            let deps = source.query(&dep)?;
             match deps.iter().map(|p| p.package_id()).max() {
                 Some(pkgid) => {
-                    let pkg = try!(source.download(pkgid));
+                    let pkg = source.download(pkgid)?;
                     Ok((pkg, Box::new(source)))
                 }
                 None => {
                     let vers_info = vers.map(|v| format!(" with version `{}`", v))
                                         .unwrap_or(String::new());
                     Err(human(format!("could not find `{}` in `{}`{}", name,
-                                      source_id, vers_info)))
+                                      source.source_id(), vers_info)))
                 }
             }
         }
         None => {
-            let candidates = try!(list_all(&mut source));
+            let candidates = list_all(&mut source)?;
             let binaries = candidates.iter().filter(|cand| {
                 cand.targets().iter().filter(|t| t.is_bin()).count() > 0
             });
             let examples = candidates.iter().filter(|cand| {
                 cand.targets().iter().filter(|t| t.is_example()).count() > 0
             });
-            let pkg = match try!(one(binaries, |v| multi_err("binaries", v))) {
+            let pkg = match one(binaries, |v| multi_err("binaries", v))? {
                 Some(p) => p,
                 None => {
-                    match try!(one(examples, |v| multi_err("examples", v))) {
+                    match one(examples, |v| multi_err("examples", v))? {
                         Some(p) => p,
                         None => bail!("no packages found with binaries or \
                                        examples"),
@@ -374,10 +401,10 @@ fn find_duplicates(dst: &Path,
 fn read_crate_list(mut file: &File) -> CargoResult<CrateListingV1> {
     (|| -> CargoResult<_> {
         let mut contents = String::new();
-        try!(file.read_to_string(&mut contents));
-        let listing = try!(toml::decode_str(&contents).chain_error(|| {
+        file.read_to_string(&mut contents)?;
+        let listing = toml::decode_str(&contents).chain_error(|| {
             internal("invalid TOML found for metadata")
-        }));
+        })?;
         match listing {
             CrateListing::V1(v1) => Ok(v1),
             CrateListing::Empty => {
@@ -391,10 +418,10 @@ fn read_crate_list(mut file: &File) -> CargoResult<CrateListingV1> {
 
 fn write_crate_list(mut file: &File, listing: CrateListingV1) -> CargoResult<()> {
     (|| -> CargoResult<_> {
-        try!(file.seek(SeekFrom::Start(0)));
-        try!(file.set_len(0));
+        file.seek(SeekFrom::Start(0))?;
+        file.set_len(0)?;
         let data = toml::encode_str::<CrateListing>(&CrateListing::V1(listing));
-        try!(file.write_all(data.as_bytes()));
+        file.write_all(data.as_bytes())?;
         Ok(())
     }).chain_error(|| {
         human("failed to write crate metadata")
@@ -402,15 +429,15 @@ fn write_crate_list(mut file: &File, listing: CrateListingV1) -> CargoResult<()>
 }
 
 pub fn install_list(dst: Option<&str>, config: &Config) -> CargoResult<()> {
-    let dst = try!(resolve_root(dst, config));
-    let dst = try!(metadata(config, &dst));
-    let list = try!(read_crate_list(dst.file()));
+    let dst = resolve_root(dst, config)?;
+    let dst = metadata(config, &dst)?;
+    let list = read_crate_list(dst.file())?;
     let mut shell = config.shell();
     let out = shell.out();
     for (k, v) in list.v1.iter() {
-        try!(writeln!(out, "{}:", k));
+        writeln!(out, "{}:", k)?;
         for bin in v {
-            try!(writeln!(out, "    {}", bin));
+            writeln!(out, "    {}", bin)?;
         }
     }
     Ok(())
@@ -420,12 +447,12 @@ pub fn uninstall(root: Option<&str>,
                  spec: &str,
                  bins: &[String],
                  config: &Config) -> CargoResult<()> {
-    let root = try!(resolve_root(root, config));
-    let crate_metadata = try!(metadata(config, &root));
-    let mut metadata = try!(read_crate_list(crate_metadata.file()));
+    let root = resolve_root(root, config)?;
+    let crate_metadata = metadata(config, &root)?;
+    let mut metadata = read_crate_list(crate_metadata.file())?;
     let mut to_remove = Vec::new();
     {
-        let result = try!(PackageIdSpec::query_str(spec, metadata.v1.keys()))
+        let result = PackageIdSpec::query_str(spec, metadata.v1.keys())?
                                         .clone();
         let mut installed = match metadata.v1.entry(result.clone()) {
             Entry::Occupied(e) => e,
@@ -467,10 +494,10 @@ pub fn uninstall(root: Option<&str>,
             installed.remove();
         }
     }
-    try!(write_crate_list(crate_metadata.file(), metadata));
+    write_crate_list(crate_metadata.file(), metadata)?;
     for bin in to_remove {
-        try!(config.shell().status("Removing", bin.display()));
-        try!(fs::remove_file(bin));
+        config.shell().status("Removing", bin.display())?;
+        fs::remove_file(bin)?;
     }
 
     Ok(())
@@ -482,7 +509,7 @@ fn metadata(config: &Config, root: &Filesystem) -> CargoResult<FileLock> {
 
 fn resolve_root(flag: Option<&str>,
                 config: &Config) -> CargoResult<Filesystem> {
-    let config_root = try!(config.get_path("install.root"));
+    let config_root = config.get_path("install.root")?;
     Ok(flag.map(PathBuf::from).or_else(|| {
         env::var_os("CARGO_INSTALL_ROOT").map(PathBuf::from)
     }).or_else(move || {
