@@ -3,32 +3,43 @@ use std::io::SeekFrom;
 use std::io::prelude::*;
 use std::path::{self, Path};
 
-use tar::{Archive, Builder, Header};
-use flate2::{GzBuilder, Compression};
 use flate2::read::GzDecoder;
+use flate2::{GzBuilder, Compression};
+use git2;
+use tar::{Archive, Builder, Header};
 
-use core::{SourceId, Package, PackageId};
+use core::{SourceId, Package, PackageId, Workspace, Source};
 use sources::PathSource;
 use util::{self, CargoResult, human, internal, ChainError, Config, FileLock};
 use ops;
 
-pub fn package(manifest_path: &Path,
-               config: &Config,
-               verify: bool,
-               list: bool,
-               metadata: bool) -> CargoResult<Option<FileLock>> {
-    let path = manifest_path.parent().unwrap();
-    let id = try!(SourceId::for_path(path));
-    let mut src = PathSource::new(path, &id, config);
-    let pkg = try!(src.root_package());
+pub struct PackageOpts<'cfg> {
+    pub config: &'cfg Config,
+    pub list: bool,
+    pub check_metadata: bool,
+    pub allow_dirty: bool,
+    pub verify: bool,
+    pub jobs: Option<u32>,
+}
 
-    if metadata {
-        try!(check_metadata(&pkg, config));
+pub fn package(ws: &Workspace,
+               opts: &PackageOpts) -> CargoResult<Option<FileLock>> {
+    let pkg = ws.current()?;
+    let config = ws.config();
+    let mut src = PathSource::new(pkg.root(),
+                                  pkg.package_id().source_id(),
+                                  config);
+    src.update()?;
+
+    if opts.check_metadata {
+        check_metadata(pkg, config)?;
     }
 
-    if list {
+    verify_dependencies(&pkg)?;
+
+    if opts.list {
         let root = pkg.root();
-        let mut list: Vec<_> = try!(src.list_files(&pkg)).iter().map(|file| {
+        let mut list: Vec<_> = src.list_files(&pkg)?.iter().map(|file| {
             util::without_prefix(&file, &root).unwrap().to_path_buf()
         }).collect();
         list.sort();
@@ -38,38 +49,39 @@ pub fn package(manifest_path: &Path,
         return Ok(None)
     }
 
+    if !opts.allow_dirty {
+        check_not_dirty(&pkg, &src)?;
+    }
+
     let filename = format!("{}-{}.crate", pkg.name(), pkg.version());
-    let dir = config.target_dir(&pkg).join("package");
-    let mut dst = match dir.open_ro(&filename, config, "packaged crate") {
-        Ok(f) => return Ok(Some(f)),
-        Err(..) => {
-            let tmp = format!(".{}", filename);
-            try!(dir.open_rw(&tmp, config, "package scratch space"))
-        }
+    let dir = ws.target_dir().join("package");
+    let mut dst = {
+        let tmp = format!(".{}", filename);
+        dir.open_rw(&tmp, config, "package scratch space")?
     };
 
     // Package up and test a temporary tarball and only move it to the final
     // location if it actually passes all our tests. Any previously existing
     // tarball can be assumed as corrupt or invalid, so we just blow it away if
     // it exists.
-    try!(config.shell().status("Packaging", pkg.package_id().to_string()));
-    try!(dst.file().set_len(0));
-    try!(tar(&pkg, &src, config, dst.file(), &filename).chain_error(|| {
+    config.shell().status("Packaging", pkg.package_id().to_string())?;
+    dst.file().set_len(0)?;
+    tar(ws, &src, dst.file(), &filename).chain_error(|| {
         human("failed to prepare local package for uploading")
-    }));
-    if verify {
-        try!(dst.seek(SeekFrom::Start(0)));
-        try!(run_verify(config, &pkg, dst.file()).chain_error(|| {
+    })?;
+    if opts.verify {
+        dst.seek(SeekFrom::Start(0))?;
+        run_verify(ws, dst.file(), opts).chain_error(|| {
             human("failed to verify package tarball")
-        }))
+        })?
     }
-    try!(dst.seek(SeekFrom::Start(0)));
+    dst.seek(SeekFrom::Start(0))?;
     {
         let src_path = dst.path();
         let dst_path = dst.parent().join(&filename);
-        try!(fs::rename(&src_path, &dst_path).chain_error(|| {
+        fs::rename(&src_path, &dst_path).chain_error(|| {
             human("failed to move temporary tarball into final location")
-        }));
+        })?;
     }
     Ok(Some(dst))
 }
@@ -101,38 +113,99 @@ fn check_metadata(pkg: &Package, config: &Config) -> CargoResult<()> {
         }
         things.push_str(&missing.last().unwrap());
 
-        try!(config.shell().warn(
-            &format!("manifest has no {things}. \
+        config.shell().warn(
+            &format!("manifest has no {things}.\n\
                     See http://doc.crates.io/manifest.html#package-metadata for more info.",
-                    things = things)))
+                    things = things))?
     }
     Ok(())
 }
 
-fn tar(pkg: &Package,
+// check that the package dependencies are safe to deploy.
+fn verify_dependencies(pkg: &Package) -> CargoResult<()> {
+    for dep in pkg.dependencies() {
+        if dep.source_id().is_path() {
+            if !dep.specified_req() {
+                bail!("all path dependencies must have a version specified \
+                       when packaging.\ndependency `{}` does not specify \
+                       a version.", dep.name())
+            }
+        }
+    }
+    Ok(())
+}
+
+fn check_not_dirty(p: &Package, src: &PathSource) -> CargoResult<()> {
+    if let Ok(repo) = git2::Repository::discover(p.root()) {
+        if let Some(workdir) = repo.workdir() {
+            debug!("found a git repo at {:?}, checking if index present",
+                   workdir);
+            let path = p.manifest_path();
+            let path = path.strip_prefix(workdir).unwrap_or(path);
+            if let Ok(status) = repo.status_file(path) {
+                if (status & git2::STATUS_IGNORED).is_empty() {
+                    debug!("Cargo.toml found in repo, checking if dirty");
+                    return git(p, src, &repo)
+                }
+            }
+        }
+    }
+
+    // No VCS recognized, we don't know if the directory is dirty or not, so we
+    // have to assume that it's clean.
+    return Ok(());
+
+    fn git(p: &Package,
+           src: &PathSource,
+           repo: &git2::Repository) -> CargoResult<()> {
+        let workdir = repo.workdir().unwrap();
+        let dirty = src.list_files(p)?.iter().filter(|file| {
+            let relative = file.strip_prefix(workdir).unwrap();
+            if let Ok(status) = repo.status_file(relative) {
+                status != git2::STATUS_CURRENT
+            } else {
+                false
+            }
+        }).map(|path| {
+            path.strip_prefix(p.root()).unwrap_or(path).display().to_string()
+        }).collect::<Vec<_>>();
+        if dirty.is_empty() {
+            Ok(())
+        } else {
+            bail!("{} dirty files found in the working directory:\n\n{}\n\n\
+                   to proceed despite this, pass the `--allow-dirty` flag",
+                  dirty.len(), dirty.join("\n"))
+        }
+    }
+}
+
+fn tar(ws: &Workspace,
        src: &PathSource,
-       config: &Config,
        dst: &File,
        filename: &str) -> CargoResult<()> {
     // Prepare the encoder and its header
     let filename = Path::new(filename);
-    let encoder = GzBuilder::new().filename(try!(util::path2bytes(filename)))
+    let encoder = GzBuilder::new().filename(util::path2bytes(filename)?)
                                   .write(dst, Compression::Best);
 
     // Put all package files into a compressed archive
     let mut ar = Builder::new(encoder);
+    let pkg = ws.current()?;
+    let config = ws.config();
     let root = pkg.root();
-    for file in try!(src.list_files(pkg)).iter() {
+    for file in src.list_files(pkg)?.iter() {
         let relative = util::without_prefix(&file, &root).unwrap();
-        try!(check_filename(relative));
-        let relative = try!(relative.to_str().chain_error(|| {
+        check_filename(relative)?;
+        let relative = relative.to_str().chain_error(|| {
             human(format!("non-utf8 path in source directory: {}",
                           relative.display()))
-        }));
-        let mut file = try!(File::open(file));
-        try!(config.shell().verbose(|shell| {
+        })?;
+        let mut file = File::open(file).chain_error(|| {
+            human(format!("failed to open for archiving: `{}`", file.display()))
+        })?;
+        config.shell().verbose(|shell| {
             shell.status("Archiving", &relative)
-        }));
+        })?;
         let path = format!("{}-{}{}{}", pkg.name(), pkg.version(),
                            path::MAIN_SEPARATOR, relative);
 
@@ -155,75 +228,77 @@ fn tar(pkg: &Package,
         // unpack the selectors 0.4.0 crate on crates.io. Either that or take a
         // look at rust-lang/cargo#2326
         let mut header = Header::new_ustar();
-        let metadata = try!(file.metadata().chain_error(|| {
+        let metadata = file.metadata().chain_error(|| {
             human(format!("could not learn metadata for: `{}`", relative))
-        }));
-        try!(header.set_path(&path).chain_error(|| {
+        })?;
+        header.set_path(&path).chain_error(|| {
             human(format!("failed to add to archive: `{}`", relative))
-        }));
+        })?;
         header.set_metadata(&metadata);
         header.set_cksum();
 
-        try!(ar.append(&header, &mut file).chain_error(|| {
+        ar.append(&header, &mut file).chain_error(|| {
             internal(format!("could not archive source file `{}`", relative))
-        }));
+        })?;
     }
-    let encoder = try!(ar.into_inner());
-    try!(encoder.finish());
+    let encoder = ar.into_inner()?;
+    encoder.finish()?;
     Ok(())
 }
 
-fn run_verify(config: &Config,
-              pkg: &Package,
-              tar: &File)
-              -> CargoResult<()> {
-    try!(config.shell().status("Verifying", pkg));
+fn run_verify(ws: &Workspace, tar: &File, opts: &PackageOpts) -> CargoResult<()> {
+    let config = ws.config();
+    let pkg = ws.current()?;
 
-    let f = try!(GzDecoder::new(tar));
+    config.shell().status("Verifying", pkg)?;
+
+    let f = GzDecoder::new(tar)?;
     let dst = pkg.root().join(&format!("target/package/{}-{}",
                                        pkg.name(), pkg.version()));
     if fs::metadata(&dst).is_ok() {
-        try!(fs::remove_dir_all(&dst));
+        fs::remove_dir_all(&dst)?;
     }
     let mut archive = Archive::new(f);
-    try!(archive.unpack(dst.parent().unwrap()));
+    archive.unpack(dst.parent().unwrap())?;
     let manifest_path = dst.join("Cargo.toml");
 
-    // When packages are uploaded to the registry, all path dependencies are
-    // implicitly converted to registry-based dependencies, so we rewrite those
+    // When packages are uploaded to a registry, all path dependencies are
+    // implicitly converted to registry dependencies, so we rewrite those
     // dependencies here.
     //
     // We also make sure to point all paths at `dst` instead of the previous
     // location that the package was originally read from. In locking the
     // `SourceId` we're telling it that the corresponding `PathSource` will be
     // considered updated and we won't actually read any packages.
-    let registry = try!(SourceId::for_central(config));
+    let cratesio = SourceId::crates_io(config)?;
     let precise = Some("locked".to_string());
-    let new_src = try!(SourceId::for_path(&dst)).with_precise(precise);
-    let new_pkgid = try!(PackageId::new(pkg.name(), pkg.version(), &new_src));
+    let new_src = SourceId::for_path(&dst)?.with_precise(precise);
+    let new_pkgid = PackageId::new(pkg.name(), pkg.version(), &new_src)?;
     let new_summary = pkg.summary().clone().map_dependencies(|d| {
         if !d.source_id().is_path() { return d }
-        d.clone_inner().set_source_id(registry.clone()).into_dependency()
+        d.clone_inner().set_source_id(cratesio.clone()).into_dependency()
     });
     let mut new_manifest = pkg.manifest().clone();
     new_manifest.set_summary(new_summary.override_id(new_pkgid));
     let new_pkg = Package::new(new_manifest, &manifest_path);
 
     // Now that we've rewritten all our path dependencies, compile it!
-    try!(ops::compile_pkg(&new_pkg, None, &ops::CompileOptions {
+    let ws = Workspace::ephemeral(new_pkg, config, None)?;
+    ops::compile_ws(&ws, None, &ops::CompileOptions {
         config: config,
-        jobs: None,
+        jobs: opts.jobs,
         target: None,
         features: &[],
         no_default_features: false,
-        spec: &[],
+        all_features: false,
+        spec: ops::Packages::Packages(&[]),
         filter: ops::CompileFilter::Everything,
-        exec_engine: None,
         release: false,
+        message_format: ops::MessageFormat::Human,
         mode: ops::CompileMode::Build,
         target_rustdoc_args: None,
         target_rustc_args: None,
-    }));
+    })?;
 
     Ok(())
 }
