@@ -48,23 +48,45 @@ pub fn prepare_target<'a, 'cfg>(cx: &mut Context<'a, 'cfg>,
                                 unit: &Unit<'a>) -> CargoResult<Preparation> {
     let _p = profile::start(format!("fingerprint: {} / {}",
                                     unit.pkg.package_id(), unit.target.name()));
-    let new = dir(cx, unit);
-    let loc = new.join(&filename(unit));
+    let new = cx.fingerprint_dir(unit);
+    let loc = new.join(&filename(cx, unit));
 
     debug!("fingerprint at: {}", loc.display());
 
-    let fingerprint = try!(calculate(cx, unit));
+    let fingerprint = calculate(cx, unit)?;
     let compare = compare_old_fingerprint(&loc, &*fingerprint);
     log_compare(unit, &compare);
 
+    // If our comparison failed (e.g. we're going to trigger a rebuild of this
+    // crate), then we also ensure the source of the crate passes all
+    // verification checks before we build it.
+    //
+    // The `Source::verify` method is intended to allow sources to execute
+    // pre-build checks to ensure that the relevant source code is all
+    // up-to-date and as expected. This is currently used primarily for
+    // directory sources which will use this hook to perform an integrity check
+    // on all files in the source to ensure they haven't changed. If they have
+    // changed then an error is issued.
+    if compare.is_err() {
+        let source_id = unit.pkg.package_id().source_id();
+        let sources = cx.packages.sources();
+        let source = sources.get(source_id).chain_error(|| {
+            internal("missing package source")
+        })?;
+        source.verify(unit.pkg.package_id())?;
+    }
+
     let root = cx.out_dir(unit);
     let mut missing_outputs = false;
     if unit.profile.doc {
         missing_outputs = !root.join(unit.target.crate_name())
                                .join("index.html").exists();
     } else {
-        for filename in try!(cx.target_filenames(unit)).iter() {
-            missing_outputs |= fs::metadata(root.join(filename)).is_err();
+        for (src, link_dst, _) in cx.target_filenames(unit)? {
+            missing_outputs |= !src.exists();
+            if let Some(link_dst) = link_dst {
+                missing_outputs |= !link_dst.exists();
+            }
         }
     }
 
@@ -126,9 +148,9 @@ impl Fingerprint {
     fn update_local(&self) -> CargoResult<()> {
         match self.local {
             LocalFingerprint::MtimeBased(ref slot, ref path) => {
-                let meta = try!(fs::metadata(path).chain_error(|| {
+                let meta = fs::metadata(path).chain_error(|| {
                     internal(format!("failed to stat `{}`", path.display()))
-                }));
+                })?;
                 let mtime = FileTime::from_last_modification_time(&meta);
                 *slot.0.lock().unwrap() = Some(mtime);
             }
@@ -172,13 +194,21 @@ impl Fingerprint {
                           a, b)
                 }
             }
-            (&LocalFingerprint::MtimeBased(ref a, ref ap),
-             &LocalFingerprint::MtimeBased(ref b, ref bp)) => {
-                let a = a.0.lock().unwrap();
-                let b = b.0.lock().unwrap();
-                if *a != *b {
-                    bail!("mtime based comopnents have changed: {:?} != {:?}, \
-                           paths are {:?} and {:?}", *a, *b, ap, bp)
+            (&LocalFingerprint::MtimeBased(ref on_disk_mtime, ref ap),
+             &LocalFingerprint::MtimeBased(ref previously_built_mtime, ref bp)) => {
+                let on_disk_mtime = on_disk_mtime.0.lock().unwrap();
+                let previously_built_mtime = previously_built_mtime.0.lock().unwrap();
+
+                let should_rebuild = match (*on_disk_mtime, *previously_built_mtime) {
+                    (None, None) => false,
+                    (Some(_), None) | (None, Some(_)) => true,
+                    (Some(on_disk), Some(previously_built)) => on_disk > previously_built,
+                };
+
+                if should_rebuild {
+                    bail!("mtime based components have changed: previously {:?} now {:?}, \
+                           paths are {:?} and {:?}",
+                          *previously_built_mtime, *on_disk_mtime, ap, bp)
                 }
             }
             _ => bail!("local fingerprint type has changed"),
@@ -215,19 +245,19 @@ impl hash::Hash for Fingerprint {
 impl Encodable for Fingerprint {
     fn encode<E: Encoder>(&self, e: &mut E) -> Result<(), E::Error> {
         e.emit_struct("Fingerprint", 6, |e| {
-            try!(e.emit_struct_field("rustc", 0, |e| self.rustc.encode(e)));
-            try!(e.emit_struct_field("target", 1, |e| self.target.encode(e)));
-            try!(e.emit_struct_field("profile", 2, |e| self.profile.encode(e)));
-            try!(e.emit_struct_field("local", 3, |e| self.local.encode(e)));
-            try!(e.emit_struct_field("features", 4, |e| {
+            e.emit_struct_field("rustc", 0, |e| self.rustc.encode(e))?;
+            e.emit_struct_field("target", 1, |e| self.target.encode(e))?;
+            e.emit_struct_field("profile", 2, |e| self.profile.encode(e))?;
+            e.emit_struct_field("local", 3, |e| self.local.encode(e))?;
+            e.emit_struct_field("features", 4, |e| {
                 self.features.encode(e)
-            }));
-            try!(e.emit_struct_field("deps", 5, |e| {
+            })?;
+            e.emit_struct_field("deps", 5, |e| {
                 self.deps.iter().map(|&(ref a, ref b)| {
                     (a, b.hash())
                 }).collect::<Vec<_>>().encode(e)
-            }));
-            try!(e.emit_struct_field("rustflags", 6, |e| self.rustflags.encode(e)));
+            })?;
+            e.emit_struct_field("rustflags", 6, |e| self.rustflags.encode(e))?;
             Ok(())
         })
     }
@@ -240,15 +270,15 @@ impl Decodable for Fingerprint {
         }
         d.read_struct("Fingerprint", 6, |d| {
             Ok(Fingerprint {
-                rustc: try!(d.read_struct_field("rustc", 0, decode)),
-                target: try!(d.read_struct_field("target", 1, decode)),
-                profile: try!(d.read_struct_field("profile", 2, decode)),
-                local: try!(d.read_struct_field("local", 3, decode)),
-                features: try!(d.read_struct_field("features", 4, decode)),
+                rustc: d.read_struct_field("rustc", 0, decode)?,
+                target: d.read_struct_field("target", 1, decode)?,
+                profile: d.read_struct_field("profile", 2, decode)?,
+                local: d.read_struct_field("local", 3, decode)?,
+                features: d.read_struct_field("features", 4, decode)?,
                 memoized_hash: Mutex::new(None),
                 deps: {
                     let decode = decode::<Vec<(String, u64)>, D>;
-                    let v = try!(d.read_struct_field("deps", 5, decode));
+                    let v = d.read_struct_field("deps", 5, decode)?;
                     v.into_iter().map(|(name, hash)| {
                         (name, Arc::new(Fingerprint {
                             rustc: 0,
@@ -262,7 +292,7 @@ impl Decodable for Fingerprint {
                         }))
                     }).collect()
                 },
-                rustflags: try!(d.read_struct_field("rustflags", 6, decode)),
+                rustflags: d.read_struct_field("rustflags", 6, decode)?,
             })
         })
     }
@@ -284,7 +314,7 @@ impl Encodable for MtimeSlot {
 
 impl Decodable for MtimeSlot {
     fn decode<D: Decoder>(e: &mut D) -> Result<MtimeSlot, D::Error> {
-        let kind: Option<(u64, u32)> = try!(Decodable::decode(e));
+        let kind: Option<(u64, u32)> = Decodable::decode(e)?;
         Ok(MtimeSlot(Mutex::new(kind.map(|(s, n)| {
             FileTime::from_seconds_since_1970(s, n)
         }))))
@@ -326,35 +356,40 @@ fn calculate<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>)
     // elsewhere. Also skip fingerprints of binaries because they don't actually
     // induce a recompile, they're just dependencies in the sense that they need
     // to be built.
-    let deps = try!(cx.dep_targets(unit));
-    let deps = try!(deps.iter().filter(|u| {
+    let deps = cx.dep_targets(unit)?;
+    let deps = deps.iter().filter(|u| {
         !u.target.is_custom_build() && !u.target.is_bin()
     }).map(|unit| {
         calculate(cx, unit).map(|fingerprint| {
             (unit.pkg.package_id().to_string(), fingerprint)
         })
-    }).collect::<CargoResult<Vec<_>>>());
+    }).collect::<CargoResult<Vec<_>>>()?;
 
     // And finally, calculate what our own local fingerprint is
     let local = if use_dep_info(unit) {
         let dep_info = dep_info_loc(cx, unit);
-        let mtime = try!(dep_info_mtime_if_fresh(&dep_info));
+        let mtime = dep_info_mtime_if_fresh(&dep_info)?;
         LocalFingerprint::MtimeBased(MtimeSlot(Mutex::new(mtime)), dep_info)
     } else {
-        let fingerprint = try!(pkg_fingerprint(cx, unit.pkg));
+        let fingerprint = pkg_fingerprint(cx, unit.pkg)?;
         LocalFingerprint::Precalculated(fingerprint)
     };
     let mut deps = deps;
     deps.sort_by(|&(ref a, _), &(ref b, _)| a.cmp(b));
+    let extra_flags = if unit.profile.doc {
+        cx.rustdocflags_args(unit)?
+    } else {
+        cx.rustflags_args(unit)?
+    };
     let fingerprint = Arc::new(Fingerprint {
-        rustc: util::hash_u64(&cx.config.rustc_info().verbose_version),
+        rustc: util::hash_u64(&cx.config.rustc()?.verbose_version),
         target: util::hash_u64(&unit.target),
         profile: util::hash_u64(&unit.profile),
         features: format!("{:?}", features),
         deps: deps,
         local: local,
         memoized_hash: Mutex::new(None),
-        rustflags: try!(cx.rustflags_args(unit)),
+        rustflags: extra_flags,
     });
     cx.fingerprints.insert(*unit, fingerprint.clone());
     Ok(fingerprint)
@@ -391,7 +426,7 @@ pub fn prepare_build_cmd<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>)
                                    -> CargoResult<Preparation> {
     let _p = profile::start(format!("fingerprint build cmd: {}",
                                     unit.pkg.package_id()));
-    let new = dir(cx, unit);
+    let new = cx.fingerprint_dir(unit);
     let loc = new.join("build");
 
     debug!("fingerprint at: {}", loc.display());
@@ -412,7 +447,7 @@ pub fn prepare_build_cmd<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>)
                 let &(ref output, ref deps) = &cx.build_explicit_deps[unit];
 
                 let local = if deps.is_empty() {
-                    let s = try!(pkg_fingerprint(cx, unit.pkg));
+                    let s = pkg_fingerprint(cx, unit.pkg)?;
                     LocalFingerprint::Precalculated(s)
                 } else {
                     let deps = deps.iter().map(|p| unit.pkg.root().join(p));
@@ -458,7 +493,7 @@ pub fn prepare_build_cmd<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>)
                 let slot = MtimeSlot(Mutex::new(None));
                 fingerprint.local = LocalFingerprint::MtimeBased(slot,
                                                                  output_path);
-                try!(fingerprint.update_local());
+                fingerprint.update_local()?;
             }
         }
         write_fingerprint(&loc, &fingerprint)
@@ -470,49 +505,44 @@ pub fn prepare_build_cmd<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>)
 fn write_fingerprint(loc: &Path, fingerprint: &Fingerprint) -> CargoResult<()> {
     let hash = fingerprint.hash();
     debug!("write fingerprint: {}", loc.display());
-    try!(paths::write(&loc, util::to_hex(hash).as_bytes()));
-    try!(paths::write(&loc.with_extension("json"),
-                      json::encode(&fingerprint).unwrap().as_bytes()));
+    paths::write(&loc, util::to_hex(hash).as_bytes())?;
+    paths::write(&loc.with_extension("json"),
+                 json::encode(&fingerprint).unwrap().as_bytes())?;
     Ok(())
 }
 
 /// Prepare work for when a package starts to build
 pub fn prepare_init(cx: &mut Context, unit: &Unit) -> CargoResult<()> {
-    let new1 = dir(cx, unit);
+    let new1 = cx.fingerprint_dir(unit);
     let new2 = new1.clone();
 
     if fs::metadata(&new1).is_err() {
-        try!(fs::create_dir(&new1));
+        fs::create_dir(&new1)?;
     }
     if fs::metadata(&new2).is_err() {
-        try!(fs::create_dir(&new2));
+        fs::create_dir(&new2)?;
     }
     Ok(())
 }
 
-/// Return the (old, new) location for fingerprints for a package
-pub fn dir(cx: &Context, unit: &Unit) -> PathBuf {
-    cx.layout(unit.pkg, unit.kind).proxy().fingerprint(unit.pkg)
-}
-
 /// Returns the (old, new) location for the dep info file of a target.
-pub fn dep_info_loc(cx: &Context, unit: &Unit) -> PathBuf {
-    dir(cx, unit).join(&format!("dep-{}", filename(unit)))
+pub fn dep_info_loc(cx: &mut Context, unit: &Unit) -> PathBuf {
+    cx.fingerprint_dir(unit).join(&format!("dep-{}", filename(cx, unit)))
 }
 
 fn compare_old_fingerprint(loc: &Path, new_fingerprint: &Fingerprint)
                            -> CargoResult<()> {
-    let old_fingerprint_short = try!(paths::read(loc));
+    let old_fingerprint_short = paths::read(loc)?;
     let new_hash = new_fingerprint.hash();
 
     if util::to_hex(new_hash) == old_fingerprint_short {
         return Ok(())
     }
 
-    let old_fingerprint_json = try!(paths::read(&loc.with_extension("json")));
-    let old_fingerprint = try!(json::decode(&old_fingerprint_json).chain_error(|| {
+    let old_fingerprint_json = paths::read(&loc.with_extension("json"))?;
+    let old_fingerprint = json::decode(&old_fingerprint_json).chain_error(|| {
         internal(format!("failed to deserialize json"))
-    }));
+    })?;
     new_fingerprint.compare(&old_fingerprint)
 }
 
@@ -543,15 +573,15 @@ fn dep_info_mtime_if_fresh(dep_info: &Path) -> CargoResult<Option<FileTime>> {
     if fs_try!(f.read_until(0, &mut cwd)) == 0 {
         return Ok(None)
     }
-    let cwd = try!(util::bytes2path(&cwd[..cwd.len()-1]));
+    let cwd = util::bytes2path(&cwd[..cwd.len()-1])?;
     let line = match f.lines().next() {
         Some(Ok(line)) => line,
         _ => return Ok(None),
     };
-    let pos = try!(line.find(": ").chain_error(|| {
+    let pos = line.find(": ").chain_error(|| {
         internal(format!("dep-info not in an understood format: {}",
                          dep_info.display()))
-    }));
+    })?;
     let deps = &line[pos + 2..];
 
     let mut paths = Vec::new();
@@ -564,9 +594,9 @@ fn dep_info_mtime_if_fresh(dep_info: &Path) -> CargoResult<Option<FileTime>> {
         while file.ends_with("\\") {
             file.pop();
             file.push(' ');
-            file.push_str(try!(deps.next().chain_error(|| {
+            file.push_str(deps.next().chain_error(|| {
                 internal(format!("malformed dep-info format, trailing \\"))
-            })));
+            })?);
         }
         paths.push(cwd.join(&file));
     }
@@ -577,9 +607,9 @@ fn dep_info_mtime_if_fresh(dep_info: &Path) -> CargoResult<Option<FileTime>> {
 fn pkg_fingerprint(cx: &Context, pkg: &Package) -> CargoResult<String> {
     let source_id = pkg.package_id().source_id();
     let sources = cx.packages.sources();
-    let source = try!(sources.get(source_id).chain_error(|| {
+    let source = sources.get(source_id).chain_error(|| {
         internal("missing package source")
-    }));
+    })?;
     source.fingerprint(pkg)
 }
 
@@ -618,7 +648,11 @@ fn mtime_if_fresh<I>(output: &Path, paths: I) -> Option<FileTime>
     }
 }
 
-fn filename(unit: &Unit) -> String {
+fn filename(cx: &mut Context, unit: &Unit) -> String {
+    // file_stem includes metadata hash. Thus we have a different
+    // fingerprint for every metadata hash version. This works because
+    // even if the package is fresh, we'll still link the fresh target
+    let file_stem = cx.file_stem(unit);
     let kind = match *unit.target.kind() {
         TargetKind::Lib(..) => "lib",
         TargetKind::Bin => "bin",
@@ -634,7 +668,7 @@ fn filename(unit: &Unit) -> String {
     } else {
         ""
     };
-    format!("{}{}-{}", flavor, kind, unit.target.name())
+    format!("{}{}-{}", flavor, kind, file_stem)
 }
 
 // The dep-info files emitted by the compiler all have their listed paths
@@ -644,12 +678,12 @@ fn filename(unit: &Unit) -> String {
 // next time.
 pub fn append_current_dir(path: &Path, cwd: &Path) -> CargoResult<()> {
     debug!("appending {} <- {}", path.display(), cwd.display());
-    let mut f = try!(OpenOptions::new().read(true).write(true).open(path));
+    let mut f = OpenOptions::new().read(true).write(true).open(path)?;
     let mut contents = Vec::new();
-    try!(f.read_to_end(&mut contents));
-    try!(f.seek(SeekFrom::Start(0)));
-    try!(f.write_all(try!(util::path2bytes(cwd))));
-    try!(f.write_all(&[0]));
-    try!(f.write_all(&contents));
+    f.read_to_end(&mut contents)?;
+    f.seek(SeekFrom::Start(0))?;
+    f.write_all(util::path2bytes(cwd)?)?;
+    f.write_all(&[0])?;
+    f.write_all(&contents)?;
     Ok(())
 }
