@@ -18,21 +18,23 @@ pub use self::sha256::Sha256;
 pub use self::to_semver::ToSemver;
 pub use self::to_url::ToUrl;
 pub use self::vcs::{GitRepo, HgRepo};
+pub use self::read2::read2;
 
 pub mod config;
 pub mod errors;
 pub mod graph;
 pub mod hex;
 pub mod important_paths;
+pub mod job;
+pub mod lev_distance;
+pub mod machine_message;
+pub mod network;
 pub mod paths;
 pub mod process_builder;
 pub mod profile;
 pub mod to_semver;
 pub mod to_url;
 pub mod toml;
-pub mod lev_distance;
-pub mod job;
-pub mod network;
 mod cfg;
 mod dependency_queue;
 mod rustc;
@@ -41,3 +43,4 @@ mod shell_escape;
 mod vcs;
 mod lazy_cell;
 mod flock;
+mod read2;
